/**
 * 특정 조건이 완료될 때까지 여러 번 계속 시도한다.
 * @param condition 조건
 * @param intervalMilliseconds 시도 간격
 * @param maxTrial 최대 시도 수
 */
export const waitUntil = async (condition: () => boolean, intervalMilliseconds = 200, maxTrial = 15): Promise<void> => {
  return await new Promise(resolve => {
    let tried = 0
    const interval = setInterval(() => {
      if ((tried > maxTrial) || condition()) {
        clearInterval(interval)
        resolve(undefined)
      }
      tried++
    }, intervalMilliseconds)
  })
}

/**
 * 일정 시간동안 기다린다.
 * @param milliSec 기다리는 시간
 */
export const waitDuring = async (milliSec = 2000): Promise<void> => {
  return new Promise((resolve) => setTimeout(resolve, milliSec))
}

export const tryUntil = (until: () => boolean | undefined,
                         tryCall: () => void,
                         intervalMilliseconds = 200,
                         maxTrial = 15) => {
  let tried = 0

  const interval = setInterval(() => {
    (tried > maxTrial) || until()
        ? clearInterval(interval)
        : tryCall?.()

    tried++
  }, intervalMilliseconds)
}